"""
A package which contains a bunch of small helpful devtools while scripting
"""

from .argsguard import args_guard
from .parse import parse_int, parse_float
from .ostools import *
from .sysargs import args_to_dictionary
from . import colorama_ext
from .progression_bar import progress_bar